/* Changelog
	1.0 - Initial Release
	1.1 - Added LoadTranslations because I forgot
	1.2 - Removed cstrike.inc (allowed access for TF2/etc games)
	1.3 - Added CVAR to enable/disable the teleportation of bots & fixed issue where the config file wouldn't get executed
	1.4 - Fixed OnConVarChange issue
	1.5 - Added menus!
	1.6 - Fixed targeting issue & Ported over to new syntax
	1.7 - Cleaned code, added new cvar, and removed autoexecconfig
	1.8 - Added HasFlags from ThatOneGuy
	1.9 - Massive changes by EDSHOT:
		- Changed plugin info.
		- Removed all convars and associated code (they arent needed).
		- Added notification convar (some people may want to have players notified).
		- Use RegAdminCmd instead of RegConsoleCmd. Immunities can be handled by SourceMod.
		- Removed increase in y axis teleportation. This causes you to get stuck in the ceiling.
		- Removed all of ThatOneGuy's code. Not needed.
		- Made bring and goto have their own functions. This way !goto and !telemenu can use the same
		  function. A plugin shouldn't have to run FakeClientCommand("sm_bring <target>");
		- Trimmed the menu code heavily. Unneeded amount of strings used for the menus, only one is
		  required for menu.AddItem since the first parameter has to be a string.
		- Made !teleport menu stay open when choosing items.
		- Added translation support.
*/

#include <sourcemod>
#include <sdktools>
#include "edlib.sp"

#pragma semicolon 1
#pragma newdecls required

#define PLUGIN_VERSION "1.9"

ConVar gcv_bNotify;
char buf[256];

public Plugin myinfo =
{
	name = "Teleport Player",
	author = "Headline, EDSHOT",
	description = "Teleport players! Original plugin by HyperKiLLeR.",
	version = PLUGIN_VERSION,
	url = "https://forums.alliedmods.net/showthread.php?t=267137"
};

public void OnPluginStart()
{
	LoadTranslations("common.phrases");
	LoadTranslations("hl_goto.phrases");
	gcv_bNotify = CreateConVar("hl_goto_notify", "1", "0: Disabled | 1: Only notify admin and player | 2: Notify all players", FCVAR_PLUGIN, true, 0.0, true, 2.0);
	RegAdminCmd("sm_goto", Command_Goto, ADMFLAG_SLAY, "Go to a player");
	RegAdminCmd("sm_bring", Command_Bring, ADMFLAG_SLAY, "Teleport a player to you");
	RegAdminCmd("sm_telemenu", Command_TeleMenu, ADMFLAG_SLAY, "Opens the Teleport Menu");
	RegAdminCmd("sm_teleport", Command_TeleMenu, ADMFLAG_SLAY, "Opens the Teleport Menu");
}

public void Bring(int client, int target)
{
	float fTeleportOrigin[3];
	float fPlayerOrigin[3];

	GetCollisionPoint(client, fPlayerOrigin);

	fTeleportOrigin[0] = fPlayerOrigin[0];
	fTeleportOrigin[1] = fPlayerOrigin[1];
	fTeleportOrigin[2] = (fPlayerOrigin[2]);

	TeleportEntity(target, fTeleportOrigin, NULL_VECTOR, NULL_VECTOR);

	if (GetConVarInt(gcv_bNotify) == 1)
	{
		Format(buf, sizeof(buf), "[SM] %T!", "PlayerHasBeenTeleported", client, target);
		PrintToChat(client, buf);
		Format(buf, sizeof(buf), "[SM] %T!", "YouHaveBeenTeleported", target, client);
		PrintToChat(target, buf);
	}
	if (GetConVarInt(gcv_bNotify) == 2)
	{
		PrintToChatAllTranslated2I("PlayerHasBeenTeleportedToPlayer", "[SM] ", "!", target, client);
	}
}

public Action Command_Bring(int client, int args)
{
	if (args != 1)
	{
		Format(buf, sizeof(buf), "[SM] %T: %s <%T>", "Usage", client, "sm_bring", "Target", client);
		ReplyToCommand(client, buf);
		return Plugin_Handled;
	}

	char sTarget[65];
	GetCmdArg(1, sTarget, sizeof(sTarget));

	char sTargetName[MAX_TARGET_LENGTH];
	int a_iTargets[MAXPLAYERS];
	int iTargetCount;
	bool bTN_ML;

	if ((iTargetCount = ProcessTargetString(sTarget, client, a_iTargets, MAXPLAYERS, COMMAND_FILTER_ALIVE, sTargetName, sizeof(sTargetName), bTN_ML)) <= 0)
	{
		Format(buf, sizeof(buf), "[SM] %T.", "NotFoundOrInvalid", client);
		ReplyToCommand(client, buf);
		return Plugin_Handled;
	}

	for (int i = 0; i < iTargetCount; i++)
	{
		int target = a_iTargets[i];
		if (IsValidClient(target))
		{
			Bring(client, target);
		}
	}

	return Plugin_Handled;
}

public void Goto(int client, int iTarget)
{
	float fTeleportOrigin[3];
	float fPlayerOrigin[3];

	GetClientAbsOrigin(iTarget, fPlayerOrigin);

	fTeleportOrigin[0] = fPlayerOrigin[0];
	fTeleportOrigin[1] = fPlayerOrigin[1];
	fTeleportOrigin[2] = fPlayerOrigin[2];

	TeleportEntity(client, fTeleportOrigin, NULL_VECTOR, NULL_VECTOR);

	if (GetConVarInt(gcv_bNotify) == 1)
	{
		Format(buf, sizeof(buf), "[SM] %T!", "YouHaveBeenTeleported", client, iTarget);
		PrintToChat(client, buf);
		Format(buf, sizeof(buf), "[SM] %T!", "PlayerHasBeenTeleported", iTarget, client);
		PrintToChat(iTarget, buf);
	}
	if (GetConVarInt(gcv_bNotify) == 2)
	{
		PrintToChatAllTranslated2I("PlayerHasBeenTeleportedToPlayer", "[SM] ", "!", client, iTarget);
	}
}

public Action Command_Goto(int client, int args)
{
	if (args != 1)
	{
		Format(buf, sizeof(buf), "[SM] %T: %s <%T>", "Usage", client, "sm_goto", "Target", client);
		ReplyToCommand(client, buf);
		return Plugin_Handled;
	}

	char sArg1[MAX_NAME_LENGTH];
	GetCmdArg(1, sArg1, sizeof(sArg1));
	int iTarget = FindTarget(client, sArg1, false, true);

	if (iTarget == -1)
	{
		Format(buf, sizeof(buf), "[SM] %T.", "NotFoundOrInvalid", client);
		ReplyToCommand(client, buf);
		return Plugin_Handled;
	}

	Goto(client, iTarget);
	return Plugin_Handled;
}

public Action Command_TeleMenu(int client, int args)
{
	if (!IsValidClient(client))
	{
		Format(buf, sizeof(buf), "[SM] %T!", "NotInGame", client);
		ReplyToCommand(client, buf);
		return Plugin_Handled;
	}

	Menu menu = new Menu(MainMenu_CallBack, (MenuAction_Select | MenuAction_End));
	Format(buf, sizeof(buf), "%T", "MainMenu", client);
	menu.SetTitle(buf);

	Format(buf, sizeof(buf), "%T", "BringPlayer", client);
	menu.AddItem("bring", buf);
	Format(buf, sizeof(buf), "%T", "GotoPlayer", client);
	menu.AddItem("goto", buf);

	menu.Display(client, MENU_TIME_FOREVER);
	return Plugin_Handled;
}

public int MainMenu_CallBack(Menu menu, MenuAction action, int param1, int param2)
{
	switch (action)
	{
		case MenuAction_Select:
		{
			switch (param2)
			{
				case 0: OpenBringMenu(param1);
				case 1: OpenGoToMenu(param1);
			}
		}
		case MenuAction_End:
		{
			delete menu;
		}
	}
}

public void OpenBringMenu(int client)
{
	Menu menu = new Menu(BringMenu_CallBack, (MenuAction_Select | MenuAction_End));
	Format(buf, sizeof(buf), "%T", "BringMenu", client);
	menu.SetTitle(buf);
	for (int i = 1; i <= MaxClients; i++)
	{
		if (IsValidClient(i))
		{
			char sItem[6];
			char sName[MAX_NAME_LENGTH];
			GetClientName(i, sName, sizeof(sName));
			IntToString(i, sItem, sizeof(sItem));
			menu.AddItem(sItem, sName);
		}
	}
	menu.ExitBackButton = true;
	menu.Display(client, MENU_TIME_FOREVER);
}

public int BringMenu_CallBack(Menu menu, MenuAction action, int param1, int param2)
{
	switch (action)
	{
		case MenuAction_Select:
		{
			char sItem[6];
			GetMenuItem(menu, param2, sItem, sizeof(sItem));
			int iPlayerIndex = StringToInt(sItem);

			if (!IsValidClient(iPlayerIndex))
			{
				Format(buf, sizeof(buf), "[SM] %T.", "InvalidClient", param1);
				ReplyToCommand(param1, buf);
			}
			else if(!IsPlayerAlive(iPlayerIndex))
			{
				Format(buf, sizeof(buf), "[SM] %T.", "PlayerIsNoLongerAlive", param1);
				ReplyToCommand(param1, buf);
			}
			else
			{
				Bring(param1, iPlayerIndex);
			}
			OpenBringMenu(param1);
		}
		case MenuAction_Cancel:
		{
			//param1 is client, param2 is cancel reason (see MenuCancel types)
			if (param2 == MenuCancel_ExitBack)
			{
				Command_TeleMenu(param1, 0);
			}
		}
		case MenuAction_End:
		{
			//param1 is MenuEnd reason, if canceled param2 is MenuCancel reason
			delete menu;
		}
	}
}

public void OpenGoToMenu(int client)
{
	Menu menu = new Menu(GoToMenu_Callback, (MenuAction_Select | MenuAction_End));
	Format(buf, sizeof(buf), "%T", "GotoMenu", client);
	menu.SetTitle(buf);
	for (int i = 1; i <= MaxClients; i++)
	{
		if (IsValidClient(i))
		{
			char sItem[6];
			char sName[MAX_NAME_LENGTH];
			GetClientName(i, sName, sizeof(sName));
			IntToString(i, sItem, sizeof(sItem));
			menu.AddItem(sItem, sName);
		}
	}
	menu.ExitBackButton = true;
	menu.Display(client, MENU_TIME_FOREVER);
}

public int GoToMenu_Callback(Menu menu, MenuAction action, int param1, int param2)
{
	switch (action)
	{
		case MenuAction_Select:
		{
			char sItem[6];
			GetMenuItem(menu, param2, sItem, sizeof(sItem));
			int iPlayerIndex = StringToInt(sItem);

			if (!IsValidClient(iPlayerIndex))
			{
				Format(buf, sizeof(buf), "[SM] %T.", "InvalidClient", param1);
				ReplyToCommand(param1, buf);
			}
			else if(!IsPlayerAlive(iPlayerIndex))
			{
				Format(buf, sizeof(buf), "[SM] %T.", "PlayerIsNoLongerAlive", param1);
				ReplyToCommand(param1, buf);
			}
			else
			{
				Goto(param1, iPlayerIndex);
			}
			OpenGoToMenu(param1);
		}
		case MenuAction_Cancel:
		{
			//param1 is client, param2 is cancel reason (see MenuCancel types)
			if (param2 == MenuCancel_ExitBack)
			{
				Command_TeleMenu(param1, 0);
			}
		}
		case MenuAction_End:
		{
			//param1 is MenuEnd reason, if canceled param2 is MenuCancel reason
			delete menu;
		}
	}
}

public void GetCollisionPoint(int client, float pos[3])
{
	float vOrigin[3];
	float vAngles[3];

	GetClientEyePosition(client, vOrigin);
	GetClientEyeAngles(client, vAngles);

	Handle trace = TR_TraceRayFilterEx(vOrigin, vAngles, MASK_SOLID, RayType_Infinite, TraceEntityFilterPlayer);

	if (TR_DidHit(trace))
	{
		TR_GetEndPosition(pos, trace);
		delete trace;
		return;
	}

	delete trace;
}

public bool TraceEntityFilterPlayer(int entity, int contentsMask)
{
	return entity > MaxClients;
}

public bool IsValidClient(int client)
{
	if ( !IsClientConnected(client) || !(1 <= client <= MaxClients) || !IsClientInGame(client) || IsClientSourceTV(client) || IsClientReplay(client) )
	{
		return false;
	}

	return true;
}
